local Verlet = {
   _NAME          = 'EXPERIMENTAL Physics Tool',
   _VERSION       = '0.1a',
   _DESCRIPTION   = 'EXPERIMENTAL Physics Tool',
   _LICENSE       = [[
   MIT License

   Copyright (c) 2017 William McCrea

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
   ]]
}

-------------------------------
-- |||HELPERS|||
-------------------------------
local pop, push = table.remove, table.insert
local lg = love.graphics
local lmr = love.math.random
local width,height = lg.getDimensions()
local function prandom(min, max) return love.math.random() * (max - min) + min end
local function dist(p1, p2) return ((p2.x-p1.x)^2+(p2.y-p1.y)^2)^0.5 end
-------------------------------
-- |||PHYSICS CLASS|||
-------------------------------
Verlet.__index = Verlet
setmetatable(Verlet, {
  __call = function (cls, ...)
    return cls.new(...)
  end,
})

function Verlet.new()
   local self = setmetatable({}, Verlet)
   self.nodes = {}
   self.links = {}

   self.gravity = 32
   self.friction = .995

   self:createNodes(32, 8)
   return self
end

function Verlet:update(dt)
   self.nodes[1].x,self.nodes[1].y = love.mouse.getPosition()

   self:updateNodes(dt)
   self:updateLinks(dt)
end

function Verlet:draw()
   --self:drawNodes()
   self:drawLinks()
end

function Verlet:createNodes(links, distance)
   local startx,starty = 32,32
   for i=1, links do
      self.nodes[i] = {x=startx, y=(i-1)*distance+starty, prevx=startx, prevy=(i-1)*distance+starty}
   end
   for i=1, links-1 do
      push(self.links, {n1=self.nodes[i], n2=self.nodes[i+1], length=distance})
   end
end

function Verlet:updateNodes(dt)
   for i,v in ipairs(self.nodes) do
      local n = self.nodes[i]
      local vx = (n.x - n.prevx) * self.friction
      local vy = (n.y - n.prevy) * self.friction

      n.prevx = n.x
      n.prevy = n.y
      n.x = n.x+vx
      n.y = n.y+vy
      n.y = n.y+self.gravity*dt
   end
end

function Verlet:updateLinks(dt)
   for i,v in ipairs(self.links) do
      local l = self.links[i]
      local dist = dist(l.n1,l.n2)
      local diff = l.length - dist
      local perc = diff / dist / 2
      local offx = (l.n2.x-l.n1.x) * perc
      local offy = (l.n2.y-l.n1.y) * perc

      if not l.n1.lock then
         l.n1.x = l.n1.x - offx
         l.n1.y = l.n1.y - offy
      end
      if not l.n2.lock then
         l.n2.x = l.n2.x + offx
         l.n2.y = l.n2.y + offy
      end
   end
end

function Verlet:drawNodes()
   for i,v in ipairs(self.nodes) do
      for j,_ in ipairs(v) do
         local n = self.nodes[i][j]
         lg.circle('fill', n.x, n.y, 2)
      end
   end
end

function Verlet:drawLinks()
   lg.setColor(255,0,0)
   for i,_ in pairs(self.links) do
      local l = self.links[i]
      lg.line(l.n1.x,l.n1.y, l.n2.x, l.n2.y)
   end
   lg.setColor(255,255,255)
end

return Verlet
